@@ -6,7 +6,7 @@
  * All content has to be passed UTF-8 encoded. The charset parameters is used
  * for the generated message only.
  *
- * Copyright 2007-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 2007-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
@@ -462,6 +462,31 @@ class Horde_Mime_Mail
         $this->_base = $basepart;
     }
 
+    /**
+     * Get the raw email data sent by this object.
+     *
+     * @param  boolean $stream  If true, return a stream resource, otherwise
+     *                          a string is returned.
+     *
+     * @return stream|string  The raw email data.
+     * @since 2.4.0
+     */
+    public function getRaw($stream = true)
+    {
+        if ($stream) {
+            $hdr = new Horde_Stream();
+            $hdr->add($this->_headers->toString(), true);
+            return Horde_Stream_Wrapper_Combine::getStream(
+                array($hdr->stream,
+                      $this->getBasePart()->toString(
+                        array('stream' => true, 'encode' => Horde_Mime_Part::ENCODE_7BIT | Horde_Mime_Part::ENCODE_8BIT | Horde_Mime_Part::ENCODE_BINARY))
+                )
+            );
+        }
+
+        return $this->_headers->toString() . $this->getBasePart()->toString();
+    }
+
     /**
      * Return the base MIME part.
      *
