@@ -3,7 +3,7 @@
  * This class provides an object-oriented representation of a MIME part
  * (defined by RFC 2045).
  *
- * Copyright 1999-2013 Horde LLC (http://www.horde.org/)
+ * Copyright 1999-2014 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
@@ -421,8 +421,8 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
     /**
      * Add to the body contents of this part.
      *
-     * @param mixed $contents   The part body. Either a string or a stream
-     *                          resource, or an array containing both.
+     * @param mixed $contents  The part body. Either a string or a stream
+     *                         resource, or an array containing both.
      *   - encoding: (string) The encoding of $contents.
      *               DEFAULT: Current transfer encoding value.
      *   - usestream: (boolean) If $contents is a stream, should we directly
@@ -931,7 +931,7 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
         $this_id = $this->getMimeId();
 
         /* Need strcmp() because, e.g., '2.0' == '2'. */
-        if (($action == 'get') && (strcmp($id, $this_id) === 0)) {
+        if (($action === 'get') && (strcmp($id, $this_id) === 0)) {
             return $this;
         }
 
@@ -939,32 +939,38 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
             $this->buildMimeIds(is_null($this_id) ? '1' : $this_id);
         }
 
-        foreach (array_keys($this->_parts) as $val) {
-            $partid = $this->_parts[$val]->getMimeId();
-            if (strcmp($id, $partid) === 0) {
+        foreach ($this->_parts as $key => $val) {
+            $partid = $val->getMimeId();
+
+            if (($match = (strcmp($id, $partid) === 0)) ||
+                (strpos($id, $partid . '.') === 0) ||
+                (strrchr($partid, '.') === '.0')) {
                 switch ($action) {
                 case 'alter':
-                    $mime_part->setMimeId($this->_parts[$val]->getMimeId());
-                    $this->_parts[$val] = $mime_part;
-                    return true;
+                    if ($match) {
+                        $mime_part->setMimeId($partid);
+                        $this->_parts[$key] = $mime_part;
+                        return true;
+                    }
+                    return $val->alterPart($id, $mime_part);
 
                 case 'get':
-                    return $this->_parts[$val];
+                    return $match
+                        ? $val
+                        : $val->getPart($id);
 
                 case 'remove':
-                    unset($this->_parts[$val]);
-                    $this->_reindex = true;
-                    return true;
+                    if ($match) {
+                        unset($this->_parts[$key]);
+                        $this->_reindex = true;
+                        return true;
+                    }
+                    return $val->removePart($id);
                 }
             }
-
-            if ((strpos($id, $partid . '.') === 0) ||
-                (strrchr($partid, '.') === '.0')) {
-                return $this->_parts[$val]->_partAction($id, $action, $mime_part);
-            }
         }
 
-        return ($action == 'get') ? null : false;
+        return ($action === 'get') ? null : false;
     }
 
     /**
@@ -1422,6 +1428,8 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
     /**
      * Determine the size of this MIME part and its child members.
      *
+     * @todo Remove $approx parameter.
+     *
      * @param boolean $approx  If true, determines an approximate size for
      *                         parts consisting of base64 encoded data.
      *
@@ -1429,24 +1437,24 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
      */
     public function getBytes($approx = false)
     {
-        $bytes = 0;
-
-        if (isset($this->_bytes)) {
-            $bytes = $this->_bytes;
-
-            /* Base64 transfer encoding is approx. 33% larger than original
-             * data size (RFC 2045 [6.8]). */
-            if ($approx && ($this->_transferEncoding == 'base64')) {
-                $bytes *= 0.75;
+        if ($this->getPrimaryType() == 'multipart') {
+            if (isset($this->_bytes)) {
+                return $this->_bytes;
             }
-        } elseif ($this->getPrimaryType() == 'multipart') {
+
+            $bytes = 0;
             reset($this->_parts);
             while (list(,$part) = each($this->_parts)) {
                 $bytes += $part->getBytes($approx);
             }
-        } elseif ($this->_contents) {
+            return $bytes;
+        }
+
+        if ($this->_contents) {
             fseek($this->_contents, 0, SEEK_END);
             $bytes = ftell($this->_contents);
+        } else {
+            $bytes = $this->_bytes;
 
             /* Base64 transfer encoding is approx. 33% larger than original
              * data size (RFC 2045 [6.8]). */
@@ -1455,12 +1463,13 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
             }
         }
 
-        return $bytes;
+        return intval($bytes);
     }
 
     /**
      * Explicitly set the size (in bytes) of this part. This value will only
      * be returned (via getBytes()) if there are no contents currently set.
+     *
      * This function is useful for setting the size of the part when the
      * contents of the part are not fully loaded (i.e. creating a
      * Horde_Mime_Part object from IMAP header information without loading the
@@ -1470,12 +1479,18 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
      */
     public function setBytes($bytes)
     {
-        $this->setDispositionParameter('size', $bytes);
+        /* Consider 'size' disposition parameter to be the canonical size.
+         * Only set bytes if that value doesn't exist. */
+        if (!$this->getDispositionParameter('size')) {
+            $this->setDispositionParameter('size', $bytes);
+        }
     }
 
     /**
      * Output the size of this MIME part in KB.
      *
+     * @todo Remove $approx parameter.
+     *
      * @param boolean $approx  If true, determines an approximate size for
      *                         parts consisting of base64 encoded data.
      *
@@ -1578,7 +1593,7 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
         } else {
             $this->setMimeId($id);
             $id = $id
-                ? $id . '.'
+                ? ((substr($id, -2) === '.0') ? substr($id, 0, -1) : ($id . '.'))
                 : '';
 
             if ($this->getType() == 'message/rfc822') {
@@ -1811,8 +1826,8 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
             $data = array($data);
         }
 
+        $append_filter = array();
         if (!empty($options['filter'])) {
-            $append_filter = array();
             foreach ($options['filter'] as $key => $val) {
                 $append_filter[] = stream_filter_append($fp, $key, STREAM_FILTER_WRITE, $val);
             }
@@ -1844,10 +1859,8 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
             $error = $e;
         }
 
-        if (!empty($options['filter'])) {
-            foreach ($append_filter as $val) {
-                stream_filter_remove($val);
-            }
+        foreach ($append_filter as $val) {
+            stream_filter_remove($val);
         }
 
         if (!empty($options['error'])) {
@@ -2012,8 +2025,12 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
         if (!$opts['forcemime'] && !$hdrs->getValue('mime-version')) {
             $ob->setType('text/plain');
 
-            if (!$opts['no_body'] && !empty($body)) {
-                $ob->setContents($body);
+            if ($len = strlen($body)) {
+                if ($opts['no_body']) {
+                    $ob->setBytes($len);
+                } else {
+                    $ob->setContents($body);
+                }
             }
 
             return $ob;
@@ -2059,10 +2076,12 @@ class Horde_Mime_Part implements ArrayAccess, Countable, Serializable
             $ob->setContentId($tmp);
         }
 
-        if (!$opts['no_body'] &&
-            !empty($body) &&
-            ($ob->getPrimaryType() != 'multipart')) {
-            $ob->setContents($body);
+        if (($len = strlen($body)) && ($ob->getPrimaryType() != 'multipart')) {
+            if ($opts['no_body']) {
+                $ob->setBytes($len);
+            } else {
+                $ob->setContents($body);
+            }
         }
 
         if (++$opts['level'] >= self::NESTING_LIMIT) {
